@@ -183,11 +183,33 @@ class MatlabSession(object):
 
         self._libmx.mxGetCell.argtypes = (POINTER(mxArray), c_size_t)
         self._libmx.mxGetCell.restype = POINTER(mxArray)
-        self._libmx.mxGetCell.errcheck = error_check
+        ### Errors has to be handled elswhere, because of NULL on uninitialized cells
+        # self._libmx.mxGetCell.errcheck = error_check
 
         self._libmx.mxSetCell.argtypes = (POINTER(mxArray), c_size_t, POINTER(mxArray))
         self._libmx.mxSetCell.restype = None
 
+        self._libmx.mxGetNumberOfFields.argtypes = (POINTER(mxArray),)
+        self._libmx.mxGetNumberOfFields.restype = c_int
+        self._libmx.mxGetNumberOfFields.errcheck = error_check
+
+        self._libmx.mxGetFieldNameByNumber.argtypes = (POINTER(mxArray), c_int)
+        self._libmx.mxGetFieldNameByNumber.restype = c_char_p
+        self._libmx.mxGetFieldNameByNumber.errcheck = error_check
+
+        self._libmx.mxGetField.argtypes = (POINTER(mxArray), c_size_t, c_char_p)
+        self._libmx.mxGetField.restype = POINTER(mxArray)
+        ### Errors has to be handled elswhere, because of NULL on uninitialized fields
+        # self._libmx.mxGetField.errcheck = error_check
+
+        # void mxSetField(mxArray *pm, mwIndex index, const char *fieldname, mxArray *pvalue);
+        self._libmx.mxSetField.argtypes = (POINTER(mxArray), c_size_t, c_char_p, POINTER(mxArray))
+        self._libmx.mxSetField.restype = None
+
+        self._libmx.mxCreateStructArray.argtypes = (c_size_t, POINTER(c_size_t), c_int, POINTER(c_char_p))
+        self._libmx.mxCreateStructArray.restype = POINTER(mxArray)
+        self._libmx.mxCreateStructArray.errcheck = error_check
+
         self._libmx.mxArrayToString.argtypes = (POINTER(mxArray),)
         self._libmx.mxArrayToString.restype = c_char_p
         self._libmx.mxArrayToString.errcheck = error_check
@@ -196,6 +218,10 @@ class MatlabSession(object):
         self._libmx.mxCreateString.restype = POINTER(mxArray)
         self._libmx.mxCreateString.errcheck = error_check
 
+        self._libmx.mxGetString.argtypes = (POINTER(mxArray), c_char_p, c_size_t)
+        self._libmx.mxGetString.restype = c_int
+        self._libmx.mxGetString.errcheck = error_check
+
         self._libmx.mxCreateNumericArray.argtypes = (c_size_t, POINTER(c_size_t), c_int, c_int)
         self._libmx.mxCreateNumericArray.restype = POINTER(mxArray)
         self._libmx.mxCreateNumericArray.errcheck = error_check
@@ -246,7 +272,10 @@ class MatlabSession(object):
 
     @property
     def output_buffer(self):
-        return self._output_buffer.value
+        if self._output_buffer is None:
+            raise RuntimeError("Output buffer was not initialized properly.")
+        else:
+            return self._output_buffer.value
 
 
     def eval(self, expression):
@@ -269,6 +298,10 @@ class MatlabSession(object):
 
         error_string = self._libmx.mxArrayToString(mxresult)
 
+        # TODO: possible memory leak, error_string should be freed
+        # (alternatively use mxGetString), perhaps error_string is
+        # handled by python (?)
+
         if error_string != "":
             raise RuntimeError("Error from MATLAB\n{0}".format(error_string))
 
@@ -301,7 +334,7 @@ class MatlabSession(object):
         imag_data = self._libmx.mxGetImagData(pm)
 
 
-        out = convert_mat_to_ndarray(self._libmx, pm)
+        out = mxarray_to_ndarray(self._libmx, pm)
 
 
         self._libmx.mxDestroyArray(pm)
@@ -316,7 +349,7 @@ class MatlabSession(object):
 
         """
 
-        pm = convert_ndarray_to_mat(self._libmx, value)
+        pm = ndarray_to_mxarray(self._libmx, value)
 
         self._libeng.engPutVariable(self._ep, name, pm)
 
@@ -343,7 +376,7 @@ class MatlabSession(object):
 
 
 def error_check(result, func, arguments):
-    if (isinstance(result, c_int) and result != 0) or (result is None):
+    if (isinstance(result, c_int) and result != 0) or (isinstance(result, POINTER(mxArray)) and not bool(result)):
         raise RuntimeError(
             "MATLAB function {func} failed ({result}) with arguments:\n{arguments}".format(
                 func=str(func),
@@ -403,25 +436,31 @@ class MatlabFunction(object):
 
         ### Left-hand side (returns) string
         nout = kwargs.get('nout', 1)
-        outs = ["OUT_{}__".format(i) for i in range(nout)]
+        outs = ["OUT{}__".format(i) for i in range(nout)]
         outs_str = ','.join(outs)
 
 
         ### Right hand side (arguments) string
         ins = []
         for i,a in enumerate(args):
-            aname = "ARG_{}__".format(i)
+            aname = "ARG{}__".format(i)
             session.put(aname, a)
             ins.append(aname)
         ins_str = ','.join(ins)
 
 
         ### MATLAB command
-        cmd = "[{outs}] = {name}({ins})".format(
-            outs=outs_str,
-            name=self.name,
-            ins=ins_str
-        )
+        if outs:
+            cmd = "[{outs}] = {name}({ins})".format(
+                outs=outs_str,
+                name=self.name,
+                ins=ins_str
+            )
+        else:
+            cmd = "{name}({ins})".format(
+                name=self.name,
+                ins=ins_str
+            )
 
 
         ### Run the function
@@ -470,7 +509,7 @@ class MatlabFunction(object):
 
 
 
-def convert_mat_to_ndarray(libmx, pm):
+def mxarray_to_ndarray(libmx, pm):
     """Convert MATLAB object `pm` to numpy equivalent."""
 
     ndims = libmx.mxGetNumberOfDimensions(pm)
@@ -483,6 +522,7 @@ def convert_mat_to_ndarray(libmx, pm):
     data = libmx.mxGetData(pm)
     imag_data = libmx.mxGetImagData(pm)
 
+
     if is_numeric:
         datasize = numelems*elem_size
 
@@ -509,6 +549,9 @@ def convert_mat_to_ndarray(libmx, pm):
 
         out = pyarray.squeeze()
 
+        if out.ndim == 0:
+            out, = np.atleast_1d(out)
+
 
     elif class_name == 'char':
         datasize = numelems + 1
@@ -533,28 +576,87 @@ def convert_mat_to_ndarray(libmx, pm):
         )
 
         out = pyarray.squeeze()
+        if out.ndim == 0:
+            out, = np.atleast_1d(out)
+
 
     elif class_name == 'cell':
         out = []
         for i in range(numelems):
             cell = libmx.mxGetCell(pm, i)
 
-            o = convert_mat_to_ndarray(libmx, cell)
+            if bool(cell):
+                o = mxarray_to_ndarray(libmx, cell)
+            else:
+                ### uninitialized cell
+                o = None
+
             out.append(o)
 
         out = np.array(out, dtype='O')
         out = out.reshape(dims[:ndims], order='F')
         out = out.squeeze()
 
+
+    elif class_name == 'struct':
+        field_num = libmx.mxGetNumberOfFields(pm)
+
+        ### Get all field names
+        field_names = []
+        for i in range(field_num):
+            field_name = libmx.mxGetFieldNameByNumber(pm, i)
+            field_names.append(field_name)
+
+        ### Get all fields
+        records = []
+        for i in range(numelems):
+            record = []
+            for field_name in field_names:
+                field = libmx.mxGetField(pm, i, field_name)
+
+                if bool(field):
+                    el = mxarray_to_ndarray(libmx, field)
+                else:
+                    ### uninitialized cell
+                    el = None
+
+                record.append(el)
+            records.append(record)
+
+
+        ### Set the dtypes right (if there is any ndarray, we want dtype=object)
+        arrays = zip(*records)
+        new_arrays = []
+        for arr in arrays:
+            contains_ndarray = np.any([isinstance(el, np.ndarray) for el in arr])
+
+            ## This loop is necessary, because np.rec.fromarrays()
+            ## cannot handle well a list of arrays of the same size
+            if contains_ndarray:
+                newarr = np.empty(len(arr), dtype='O')
+                for i,a in enumerate(arr):
+                    newarr[i] = a
+
+            else:
+                newarr = np.array(arr)
+
+            new_arrays.append(newarr)
+
+        out = np.rec.fromarrays(new_arrays, names=field_names)
+        out = out.reshape(dims[:ndims], order='F')
+        out = out.squeeze()
+
+
     else:
         raise NotImplementedError('{}-arrays are not supported'.format(class_name))
 
+
     return out
 
 
 
 
-def convert_ndarray_to_mat(libmx, arr):
+def ndarray_to_mxarray(libmx, arr):
 
     if isinstance(arr, str):
         pm = libmx.mxCreateString(arr)
@@ -586,6 +688,7 @@ def convert_ndarray_to_mat(libmx, arr):
             np_data = arr.imag.tostring('F')
             ctypes.memmove(mat_data, np_data, len(np_data))
 
+
     elif isinstance(arr, np.ndarray) and arr.dtype.kind == 'b':
         dim = arr.ctypes.shape_as(c_size_t)
 
@@ -595,25 +698,37 @@ def convert_ndarray_to_mat(libmx, arr):
         np_data = arr.real.tostring('F')
         ctypes.memmove(mat_data, np_data, len(np_data))
 
-    elif isinstance(arr, np.ndarray) and arr.dtype.kind == 'O':
+
+    elif isinstance(arr, np.ndarray) and arr.dtype.kind in ('O', 'S'):
         dim = arr.ctypes.shape_as(c_size_t)
 
         pm = libmx.mxCreateCellArray(arr.ndim, dim)
 
         for i,el in enumerate(arr.flatten('F')):
-            p = convert_ndarray_to_mat(libmx, el)
+            p = ndarray_to_mxarray(libmx, el)
             libmx.mxSetCell(pm, i, p)
 
-    # elif pyvariable.dtype.kind =='S':
-    #     dim = pyvariable.ctypes.shape_as(c_size_t)
-    #     mx = libmx.mxCreateNumericArray(c_size_t(pyvariable.ndim),
-    #                                           dim)
-    #     data_old = libmx.mxGetData(mx)
-    #     datastring = pyvariable.tostring('F')
-    #     n_datastring = len(datastring)
-    #     memmove(data_old,datastring,n_datastring)
-    # elif pyvariable.dtype.kind =='O':
-    #     raise NotImplementedError('Object arrays are not supported')
+
+    elif isinstance(arr, np.ndarray) and len(arr.dtype) > 0:
+        dim = arr.ctypes.shape_as(c_size_t)
+
+        name_num = len(arr.dtype.names)
+
+        names_p = (c_char_p*name_num)(*[c_char_p(name) for name in arr.dtype.names])
+
+        pm = libmx.mxCreateStructArray(
+            arr.ndim,
+            dim,
+            name_num,
+            names_p,
+        )
+
+        for i,record in enumerate(arr.flatten('F')):
+            for name in arr.dtype.names:
+                el = record[name]
+                p = ndarray_to_mxarray(libmx, el)
+
+                libmx.mxSetField(pm, i, name, p)
 
     elif isinstance(arr, np.ndarray):
         raise NotImplementedError('Type {} not supported.'.format(arr.dtype))
